package com.tzld.longarticle.recommend.server.service.recommend;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishContentStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.longArticle.ArticleVideoAuditStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.longArticle.ArticleVideoBadStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ContentPoolEnum;
import com.tzld.longarticle.recommend.server.mapper.aigc.PublishContentMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.VideoPoolAuditMapper;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.PublishSingleVideoSource;
import com.tzld.longarticle.recommend.server.model.param.videoAudit.*;
import com.tzld.longarticle.recommend.server.model.vo.VideoPoolAuditListVO;
import com.tzld.longarticle.recommend.server.repository.longArticle.PublishSingleVideoSourceRepository;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.RedisUtil;
import com.tzld.longarticle.recommend.server.util.page.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service
@Slf4j
public class VideoPoolAuditService {

    @Autowired
    private VideoPoolAuditMapper videoPoolAuditMapper;
    @Autowired
    private PublishContentMapper publishContentMapper;
    @Autowired
    private PublishSingleVideoSourceRepository videoSourceRepository;
    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Value("${cdnUrl:https://rescdn.piaoquantv.com/}")
    private String cdnUrl;

    @Value("${videoAudit.poolLevel.sort:}")
    private String poolLevelDesc;

    @ApolloJsonValue("${daily.video.audit.pool.count.config:{}}")
    private Map<String, Integer> dailyAuditPoolCount;

    public Page<VideoPoolAuditListVO> list(VideoPoolAuditListParam param) {
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        int count = videoPoolAuditMapper.articleVideoAuditListCount(param.getContentId(), param.getStatus(),
                param.getTitle(), param.getAuditAccount(), param.getFlowPoolLevel(), param.getAuditTimestamp());
        List<PublishSingleVideoSource> list = videoPoolAuditMapper.articleVideoAuditList(param.getContentId(),
                param.getStatus(), param.getTitle(), param.getAuditAccount(), param.getFlowPoolLevel()
                , param.getAuditTimestamp(), offset, param.getPageSize(), poolLevelDesc);
        List<VideoPoolAuditListVO> result = buildVideoPoolAuditListVO(list);
        Page<VideoPoolAuditListVO> page = new Page<>(param.getPageNum(), param.getPageSize());
        page.setTotalSize(count);
        page.setObjs(result);
        return page;
    }

    private List<VideoPoolAuditListVO> buildVideoPoolAuditListVO(List<PublishSingleVideoSource> list) {
        List<VideoPoolAuditListVO> result = new ArrayList<>();
        if (CollectionUtils.isEmpty(list)) {
            return result;
        }
        for (PublishSingleVideoSource item : list) {
            VideoPoolAuditListVO vo = new VideoPoolAuditListVO();
            vo.setContentId(item.getContentTraceId());
            vo.setTitle(item.getArticleTitle());
            vo.setStatus(item.getVideoPoolAuditStatus());
//            vo.setSynopsis(item.getSummaryText());
            vo.setFlowPoolLevel(ContentPoolEnum.from(item.getFlowPoolLevel()).getDescription());
            vo.setCrawlerTimestamp(item.getCrawlerTimestamp() * 1000);
            vo.setAuditAccount(item.getVideoPoolAuditAccount());
            if (Objects.nonNull(item.getVideoPoolAuditTimestamp())) {
                vo.setAuditTimestamp(item.getVideoPoolAuditTimestamp());
            }
            VideoPoolAuditListVO.VideoItem videoItem = new VideoPoolAuditListVO.VideoItem();
            videoItem.setVideoId(item.getContentTraceId());
            videoItem.setTitle(item.getMiniProgramTitle());
            videoItem.setVideoUrl(cdnUrl + item.getVideoOssPath());
            videoItem.setCoverUrl(item.getCoverUrl());
            vo.setVideo(videoItem);
            result.add(vo);
        }
        return result;
    }

    public Page<VideoPoolAuditListVO> next(VideoPoolAuditListParam param) {
        Page<VideoPoolAuditListVO> result = new Page<>();
        String lockKey = "video-pool-audit-lock";
        String requestId = UUID.randomUUID().toString();
        int retryCount = 0;
        boolean lockAcquired = false;

        // 尝试获取分布式锁，最多重试10次，每次间隔200ms
        while (retryCount < 10) {
            lockAcquired = redisUtil.tryAcquireLock(lockKey, requestId);
            if (lockAcquired) {
                break;
            }
            retryCount++;
            try {
                Thread.sleep(200); // 等待200ms后重试
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                return result; // 返回空结果
            }
        }

        if (!lockAcquired) {
            return result; // 返回空结果
        }
        long now = System.currentTimeMillis();
        String id;
        PublishSingleVideoSource obj = null;
        String inAuditListRedisKey = "video-pool-in-audit-list";
        try {
            String dateStr = DateUtils.getCurrentDateStr("yyyyMMdd");
            String auditQueueRedisKey = "video-pool-audit-queue-" + dateStr;
            // 从待审核队列中获取数据，如未获取到则从数据库查询一条
            Long size = redisTemplate.opsForZSet().size(auditQueueRedisKey);
            if (Objects.isNull(size) || size == 0) {
                List<String> entries = redisTemplate.opsForList().range(inAuditListRedisKey, 0, -1);
                List<String> excludeContentIds = new ArrayList<>();
                if (CollectionUtils.isNotEmpty(entries)) {
                    excludeContentIds = entries.stream().map(o -> {
                        JSONObject json = JSONObject.parseObject(o);
                        return json.getString("id");
                    }).collect(Collectors.toList());
                }
                obj = videoPoolAuditMapper.articleVideoAuditNext(param.getContentId(),
                        param.getStatus(), param.getTitle(), param.getAuditAccount(), null, excludeContentIds);
            } else {
                // 从待审核队列中获取数据, 如获取到的内容已审核，则重新获取
                while (true) {
                    Set<String> ids = redisTemplate.opsForZSet().reverseRangeByScore(auditQueueRedisKey, 0, 100, 0, 1);
                    if (CollectionUtils.isNotEmpty(ids)) {
                        id = ids.iterator().next();
                        redisTemplate.opsForZSet().remove(auditQueueRedisKey, id);
                        obj = videoSourceRepository.getByContentTraceId(id);
                        if (obj.getVideoPoolAuditStatus() == ArticleVideoAuditStatusEnum.WAITING.getCode()) {
                            break;
                        }
                    } else {
                        break;
                    }
                }
            }
        } finally {
            // 释放锁
            redisUtil.releaseLock(lockKey, requestId);
        }
        if (Objects.isNull(obj)) {
            return result;
        }
        // 添加到超时队列
        JSONObject json = new JSONObject();
        json.put("id", obj.getContentTraceId());
        json.put("timestamp", now + (10 * 60 * 1000));
        redisTemplate.opsForList().rightPush(inAuditListRedisKey, json.toJSONString());
        // 填充数据
        List<VideoPoolAuditListVO> list = buildVideoPoolAuditListVO(Collections.singletonList(obj));
        result.setObjs(list);
        return result;
    }

    public void auditArticle(ArticleAuditParam param) {
        PublishSingleVideoSource videoAudit = videoSourceRepository.getByContentTraceId(param.getContentId());
        Long now = System.currentTimeMillis();
        if (param.getStatus().equals(ArticleVideoAuditStatusEnum.REJECT.getCode())) {
            videoAudit.setBadStatus(ArticleVideoBadStatusEnum.TITLE_AIGC_REJECT_STATUS.getCode());
        } else {
            videoAudit.setBadStatus(ArticleVideoBadStatusEnum.TITLE_DEFAULT_STATUS.getCode());
        }
        videoAudit.setVideoPoolAuditStatus(param.getStatus());
        videoAudit.setVideoPoolAuditAccount(param.getAuditAccount());
        videoAudit.setVideoPoolAuditTimestamp(now);
        videoSourceRepository.save(videoAudit);
        if (param.getStatus() == 2) {
            // 审核不通过
            auditArticleReject(videoAudit);
        } else {
            // 审核通过
            auditArticlePass(videoAudit);
        }
        // 当日审核数+1
        addAuditCount(ContentPoolEnum.from(videoAudit.getFlowPoolLevel()).getContentPool());
    }

    private void addAuditCount(String poolLevel) {
        String dateStr = DateUtils.getCurrentDateStr("yyyyMMdd");
        String key = "video_audit_count_" + dateStr + "_" + poolLevel;
        int totalCount = Integer.parseInt(Optional.ofNullable(redisTemplate.opsForValue().get(key)).orElse("0"));
        redisTemplate.opsForValue().set(key, String.valueOf(totalCount + 1), 24, TimeUnit.HOURS);
    }

    private void auditArticlePass(PublishSingleVideoSource videoAudit) {
        // publish_content
        publishContentMapper.updatePublishContentStatus(videoAudit.getContentTraceId(),
                PublishContentStatusEnum.waiting_publish.getVal(),
                PublishContentStatusEnum.gzh_article_deprecated.getVal());
    }

    private void auditArticleReject(PublishSingleVideoSource videoAudit) {
        // publish_content
        publishContentMapper.updatePublishContentStatus(videoAudit.getContentTraceId(),
                PublishContentStatusEnum.gzh_article_deprecated.getVal(),
                PublishContentStatusEnum.waiting_publish.getVal());
    }

    public List<Object> getFilterValue(AuditFilterParam param) {
        String itemName = param.getItemName();
        if ("contentId".equals(itemName) || "title".equals(itemName) || "auditAccount".equals(itemName)) {
            return videoPoolAuditMapper.searchFilterValueByItemName(itemName, param.getSearchKeyword())
                    .stream().collect(Collectors.toList());
        } else {
            return new ArrayList<>();
        }
    }

    public void updateArticleTitle(ArticleTitleUpdateParam param) {
        long now = System.currentTimeMillis();
        videoPoolAuditMapper.updateArticleTitle(param, now);
    }

    public void updateVideoTitle(VideoTitleUpdateParam param) {
        long now = System.currentTimeMillis();
        videoPoolAuditMapper.updateVideoTitle(param, now);
    }

    @XxlJob("shuffleVideoPoolAuditGroup")
    public ReturnT<String> shuffleVideoPoolAuditGroup(String param) {
        List<String> auditUser = Arrays.asList("a", "b", "c", "d", "e", "f", "g", "h", "i", "j");
        List<PublishSingleVideoSource> contentIds = videoSourceRepository.getByVideoPoolAuditStatus(ArticleVideoAuditStatusEnum.WAITING.getCode());
        for (int i = 0; i < contentIds.size(); i++) {
            int per = i % auditUser.size();
            PublishSingleVideoSource audit = contentIds.get(i);
            audit.setVideoPoolAuditAccount(auditUser.get(per));
            videoSourceRepository.save(audit);
        }
        return ReturnT.SUCCESS;
    }

    @XxlJob("videoPoolAuditQueueJob")
    public ReturnT<String> videoPoolAuditQueueJob(String param) {
        String lockKey = "video-pool-audit-queue-job-lock";
        String requestId = UUID.randomUUID().toString();
        Boolean lockAcquired = redisUtil.tryAcquireLock(lockKey, requestId);
        if (!lockAcquired) {
            return ReturnT.SUCCESS;
        }
        try {
            Long now = System.currentTimeMillis();
            String dateStr = DateUtils.getCurrentDateStr("yyyyMMdd");
            String inAuditListRedisKey = "video-pool-in-audit-list";
            String auditQueueRedisKey = "video-pool-audit-queue-" + dateStr;
            // 判断是否审核超时，重新加入待审核队列
            while (true) {
                List<String> inAuditList = redisTemplate.opsForList().range(inAuditListRedisKey, 0, 0);
                if (CollectionUtils.isEmpty(inAuditList)) {
                    break;
                }
                JSONObject firstObj = JSONObject.parseObject(inAuditList.get(0));
                if (now > firstObj.getLong("timestamp")) {
                    redisTemplate.opsForList().leftPop(inAuditListRedisKey);
                    String id = firstObj.getString("id");
                    PublishSingleVideoSource videoAudit = videoSourceRepository.getByContentTraceId(id);
                    if (videoAudit.getVideoPoolAuditStatus() == ArticleVideoAuditStatusEnum.WAITING.getCode()) {
                        ContentPoolEnum poolEnum = ContentPoolEnum.from(videoAudit.getFlowPoolLevel());
                        redisTemplate.opsForZSet().add(auditQueueRedisKey, id, poolEnum.getWeight());
                    }
                } else {
                    break;
                }
            }
            List<String> excludeContentIds = new ArrayList<>();
            // 审核中列表获取，按内容池分组
            List<String> inAuditList = redisTemplate.opsForList().range(inAuditListRedisKey, 0, -1);
            Map<String, List<PublishSingleVideoSource>> inAuditListPoolCountMap = new HashMap<>();
            if (CollectionUtils.isNotEmpty(inAuditList)) {
                List<String> inAuditListIds = inAuditList.stream().map(item -> JSONObject.parseObject(item).getString("id")).collect(Collectors.toList());
                List<PublishSingleVideoSource> auditQueueList = videoSourceRepository.getByContentTraceIdIn(new ArrayList<>(inAuditListIds));
                inAuditListPoolCountMap = auditQueueList.stream().collect(Collectors.groupingBy(item -> ContentPoolEnum.from(item.getFlowPoolLevel()).getContentPool()));
                excludeContentIds.addAll(inAuditListIds);
            }
            // 待审核列表获取，按内容池分组
            Set<String> auditQueueIds = redisTemplate.opsForZSet().rangeByScore(auditQueueRedisKey, 0, 100);
            Map<String, List<PublishSingleVideoSource>> auditQueuePoolCountMap = new HashMap<>();
            if (CollectionUtils.isNotEmpty(auditQueueIds)) {
                List<PublishSingleVideoSource> auditQueueList = videoSourceRepository.getByContentTraceIdIn(new ArrayList<>(auditQueueIds));
                auditQueuePoolCountMap = auditQueueList.stream().collect(Collectors.groupingBy(item -> ContentPoolEnum.from(item.getFlowPoolLevel()).getContentPool()));
                excludeContentIds.addAll(auditQueueIds);
            }
            // 每日配置发送量不足添加
            for (Map.Entry<String, Integer> entry : dailyAuditPoolCount.entrySet()) {
                ContentPoolEnum poolEnum = ContentPoolEnum.from(entry.getKey());
                String key = "video_audit_count_" + dateStr + "_" + poolEnum.getContentPool();
                int totalCount = Integer.parseInt(Optional.ofNullable(redisTemplate.opsForValue().get(key)).orElse("0"));
                if (totalCount < entry.getValue()) {
                    List<PublishSingleVideoSource> auditQueueList = auditQueuePoolCountMap.getOrDefault(poolEnum.getContentPool(), new ArrayList<>());
                    List<PublishSingleVideoSource> inAuditListPool = inAuditListPoolCountMap.getOrDefault(poolEnum.getContentPool(), new ArrayList<>());
                    int needCount = entry.getValue() - (totalCount + auditQueueList.size() + inAuditListPool.size());
                    if (needCount > 0) {
                        List<PublishSingleVideoSource> addList = videoPoolAuditMapper.articleVideoWatingAuditList(
                                Arrays.asList(ArticleVideoAuditStatusEnum.WAITING.getCode()),
                                Arrays.asList(poolEnum.getValue()), excludeContentIds, needCount);
                        if (CollectionUtils.isNotEmpty(addList)) {
                            for (PublishSingleVideoSource item : addList) {
                                redisTemplate.opsForZSet().add(auditQueueRedisKey, item.getContentTraceId(), poolEnum.getWeight());
                                excludeContentIds.add(item.getContentTraceId());
                            }
                        }
                    }
                }
            }
            // 待发布内容不足添加
            Long auditQueueSize = redisTemplate.opsForZSet().size(auditQueueRedisKey);
            if (Objects.isNull(auditQueueSize) || auditQueueSize < 20) {
                List<PublishSingleVideoSource> list = videoPoolAuditMapper.articleVideoWatingAuditList(
                        Arrays.asList(ArticleVideoAuditStatusEnum.WAITING.getCode()), null, excludeContentIds, 40);
                if (CollectionUtils.isNotEmpty(list)) {
                    for (PublishSingleVideoSource item : list) {
                        ContentPoolEnum poolEnum = ContentPoolEnum.from(item.getFlowPoolLevel());
                        redisTemplate.opsForZSet().add(auditQueueRedisKey, item.getContentTraceId(), poolEnum.getWeight());
                    }
                }
            }
            redisTemplate.expire(auditQueueRedisKey, 12, TimeUnit.HOURS);
        } finally {
            redisUtil.releaseLock(lockKey, requestId);
        }
        return ReturnT.SUCCESS;
    }
}
